"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.getFetchArgs = exports.CSRF_HEADER_COOKIE_NAME = exports.CSRF_COOKIE_NAME = void 0;

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _jsCookie = _interopRequireDefault(require("js-cookie"));

var _rison = _interopRequireDefault(require("rison"));

var _fetchUtils = require("../util/fetchUtils");

var CSRF_COOKIE_NAME = process.env.NODE_ENV === 'production' ? 'x-mwp-csrf' : 'x-mwp-csrf_dev';
exports.CSRF_COOKIE_NAME = CSRF_COOKIE_NAME;
var CSRF_HEADER_COOKIE_NAME = "".concat(CSRF_COOKIE_NAME, "-header");
/*
 * rison serialization fails for unserializable data, including params with
 * `undefined` values. This if/else will log an error in the browser in dev
 * in order to catch mistakes, and force-clean the data in prod to avoid
 * client app crashes. In many cases, `undefined` params will not result in
 * invalid return values, but they should be cleaned up to avoid uncertainty
 *
 * This if/else will be simplified to only contain the correct block in the
 * production bundle
 */

exports.CSRF_HEADER_COOKIE_NAME = CSRF_HEADER_COOKIE_NAME;

var makeSerializable = function makeSerializable(queries) {
  if (process.env.NODE_ENV === 'production') {
    // quick-and-cheap object cleanup for serialization. This will remove
    // undefined and unserializable values (e.g. functions)
    return JSON.parse(JSON.stringify(queries));
  } else {
    if (queries.length > 1 && queries.some(function (_ref) {
      var params = _ref.params;
      return params instanceof FormData;
    })) {
      throw new Error('POST queries with FormData cannot be batched', '- dispatch each one individually');
    }

    try {
      _rison.default.encode_array(queries);
    } catch (err) {
      console.error(err);
      console.error('Problem encoding queries', '- please ensure that there are no undefined params', JSON.stringify(queries, null, 2));
    }
  }

  return queries;
};
/**
 * Build the arguments for the `fetch` call to the app server that will
 * contain the batched queries
 *
 * @param {String} apiUrl the general-purpose endpoint for API calls to the
 *   application server
 * @param {Array} queries the queries to send - must all use the same `method`
 * @param {Object} meta additional characteristics of the request, e.g.
 *   click tracking data
 * @return {Object} { url, config } arguments for a fetch call
 */


var getFetchArgs = function getFetchArgs(apiUrl, queries, meta, activityInfo) {
  var headers = {};
  var method = ((queries[0].meta || {}).method || 'GET'). // fallback to 'get'
  toUpperCase(); // must be upper case - requests can fail silently otherwise

  var hasBody = method === 'POST' || method === 'PATCH' || method === 'PUT';
  var isFormData = queries[0].params instanceof FormData;
  var isDelete = method === 'DELETE';
  var pageSearchParams = new URLSearchParams(window.location.search);
  var searchParams = new URLSearchParams();
  searchParams.append('queries', _rison.default.encode_array(makeSerializable(queries)));

  if (pageSearchParams.has('__set_geoip')) {
    searchParams.append('__set_geoip', pageSearchParams.get('__set_geoip'));
  }

  if (meta) {
    var onSuccess = meta.onSuccess,
        onError = meta.onError,
        promise = meta.promise,
        request = meta.request,
        metadata = (0, _objectWithoutProperties2.default)(meta, ["onSuccess", "onError", "promise", "request"]); // send other metadata in searchParams

    var encodedMetadata = metadata && _rison.default.encode_object(metadata);

    if (encodedMetadata) {
      // send other metadata in searchParams
      searchParams.append('metadata', encodedMetadata);
    }
  }

  if (!isFormData) {
    // need to manually specify content-type for any non-multipart request
    headers['content-type'] = hasBody ? 'application/x-www-form-urlencoded' : 'application/json';
  }

  if (hasBody || isDelete) {
    headers[CSRF_COOKIE_NAME] = _jsCookie.default.get(CSRF_HEADER_COOKIE_NAME);
  }

  if (activityInfo) {
    headers['x-meetup-activity'] = new URLSearchParams(activityInfo).toString();
  }

  var config = {
    method: method,
    headers: headers,
    credentials: 'same-origin' // allow response to set-cookies

  };

  if (hasBody) {
    config.body = isFormData ? queries[0].params : searchParams.toString();
  }

  var useQueryString = isFormData || !hasBody;
  var url = useQueryString ? "".concat(apiUrl, "?").concat(searchParams) : apiUrl;
  return {
    url: url,
    config: config
  };
};

exports.getFetchArgs = getFetchArgs;

var _fetchQueryResponse = function _fetchQueryResponse(_ref2) {
  var apiUrl = _ref2.apiUrl,
      queries = _ref2.queries,
      meta = _ref2.meta,
      activityInfo = _ref2.activityInfo;

  if (queries.length === 0) {
    // no queries => no responses (no need to fetch)
    return Promise.resolve({
      responses: []
    });
  }

  var _getFetchArgs = getFetchArgs(apiUrl, queries, meta, activityInfo),
      url = _getFetchArgs.url,
      config = _getFetchArgs.config;

  return fetch(url, config).catch(function (err) {
    console.error(err);
    console.error('App server API fetch error');
    throw err; // handle the error upstream
  }).then(function (queryResponse) {
    return queryResponse.json().catch(function (err) {
      console.error(err);
      console.error('App server API response JSON error');
      throw err; // handle the error upstream
    });
  });
};
/**
 * Wrapper around `fetch` to send an array of queries to the server and organize
 * the responses.
 *
 * **IMPORTANT**: This function should _only_ be called from the browser. The
 * server should never need to call itself over HTTP
 *
 * @param {String} apiUrl the general-purpose endpoint for API calls to the
 *   application server
 * @param {Array} queries the queries to send - must all use the same `method`
 * @param {Object} meta additional characteristics of the request, e.g.
 *   click tracking data
 * @param {Object} activityInfo optional map of additional activity record data
 *   to pass along with fetch request
 * @return {Promise} resolves with a `{queries, responses}` object
 */


var fetchQueries = function fetchQueries(apiUrl, member) {
  return function (queries, meta, activityInfo) {
    if (typeof window === 'undefined' && typeof test === 'undefined' // not in browser // not in testing env (global set by Jest)
    ) {
        throw new Error('fetchQueries was called on server - cannot continue');
      }

    var authedQueries = (0, _fetchUtils.getAuthedQueryFilter)(member);
    var validQueries = queries.filter(authedQueries);
    return _fetchQueryResponse({
      apiUrl: apiUrl,
      queries: validQueries,
      meta: meta,
      activityInfo: activityInfo
    }).then((0, _fetchUtils.parseQueryResponse)(validQueries));
  };
};

var _default = fetchQueries;
exports.default = _default;
//# sourceMappingURL=fetchQueries.js.map